/*

	An Adobe Illustrator L-System through Scriptographer v1.0
	by Håkan Lundgren
	© 2011 Monovektor.com
	
	
	A quick introduction of L-Systems:
	http://en.wikipedia.org/wiki/Lsystem
	
	
	
	
	
	The L-System object takes two arguments: angle & axiom.
		- angle: a number by how much a full circle is divided (ie. angle: 4 = 90°).
		- axiom: the starting value (a string) of the system.
	
	
	
	The rules have two or three arguments depending on the rule: rule, grammar & [probabilities]
		- rule: a single letter/char string.
		- grammar: a string (or an array of strings IF probabilities are used) that replace the rule char.
		- probabilities: an array of numbers indicating the probability of its corresponding grammar. For instance,
		  if grammar is an array of two or more strings probabilities would then be an array of equal length such:
	
			  rule: 'F'
			  grammar: ['F[+F]-F', 'F[+F+F]-']
			  probabilities: [0.2, 0.8]
			  
			  Would look like this:
			  	new Rule('F', ['F[+F]-F', 'F[+F+F]-'], [0.2, 0.8])
	
		  Whenever an F occurs it will be substituted by either one of the grammars according to each grammars probability.
		  The first one will only be picked about 20% of the times. It is important that the numbers in the probabilities
		  array adds up to exactly 1 (one). A L-System with this particular feature is called a stochastic L-system.
		- If however no probabilities are used (as in most cases) only one rule and grammar string are needed.
		- A grammar cannot have an empty string (''), if no substitution is to be made use a space (' ') instead.
	
	
	
	The draw function also takes two or three arguments.
		- iter: number of times to iterate through the initial axiom. The alphabet grows exponentially so beware of high
		  numbers, usually a value between 4 - 9 is sufficient. Sometimes as low as 2 - 6.
		- lineLength: the initial length of a line meassured in points. Most grammars never change the length at all.
		- lineChange: a multiplier (and divider) by how much a line will grow och shrink. ~0.9-1.1 seem to yield the best
		  results as any lower/higher value will radically change the length.
	
	
	
	A typical L-System will look like this:
		var lsystem = new Lsystem(8, '++FX')
		lsystem.rules.push(new Rule('X', '<<<<[-FX]+FX'))
			::
		lsystem.rules.push(...More rules, if any)
			::
		lsystem.draw(12, 40, 0.9)
	
	
	
	The constants used are:
		F	Moves forward one step (length defined by the lineLength variable) and draws a line.
		f	Moves forward one step without drawing a line.
		+	Changes the angle in a counter clockwise direction.
		-	Changes the angle in a clockwise direction.
		|	Turns 180°
		<	Multiplies the current line length by a number defined by the lineChange variable
		>	Divide the current line length by a number defined by the lineChange variable
		[	Create a branch, similar to the array method push()
		]	End branch and return to its parrent, similar to the array method pop()
		
	More constants may come later.
	
	
	
	At the bottom is a collection of nice looking grammars I have found all over the place.
	More reading about L-Systems - written by the inventor of it - can be found in the book The Algorithmic Beauty of
	Plants available for download as a high quality PDF here:
	http://algorithmicbotany.org/papers/abop/abop.pdf
	
*/


var Lsystem = function(angle, axiom){
			this.angle = angle
			this.axiom = axiom
			this.rules = []
			this.alphabet = axiom
}


var Rule = function(rule, grammar, probabilities){
			this.name = rule
			this.grammar = grammar
			this.probability = []

			// Goes through every value to make an interval out of them
			if(probabilities){

					var previousValue = nextValue = 0

					for(var i = 0; i < probabilities.length; i++){
							nextValue = previousValue + probabilities[i]
							this.probability.push(nextValue)
							previousValue = nextValue
					}

			} else { this.probability = [1] }


			this.getRule = function(){

					var randomRule = Math.random()
					var counter = 0

					while(randomRule > this.probability[counter]){
							counter++
					}

					return this.probability.length > 1 ? this.grammar[counter] : this.grammar
			}
}


Lsystem.prototype.generateAlphabet = function(iterations){
			var temp = ''

			// Iterator loop
			for(var i = 0; i < iterations; i++){
					// Loop through all letters
					for(var j = 0; j < this.alphabet.length; j++){

							var W = this.alphabet.charAt(j)

							// Find rule
							for(k in this.rules){
								if(this.rules[k].name == W){
									W = this.rules[k].getRule()
								}
							}

							// Replace letter with rule
							temp += W
					}

					// Change alphabet
					this.alphabet = temp
					temp = ''		
			}

			return this.alphabet
}



Lsystem.prototype.draw = function(iter, lineLength, lineChange){
	
			this.generateAlphabet(iter)

			var vector = new Point({ length: lineLength, angle: 0 })
			var branches = []
			var path = new Path()
			var currentPosition = new Point(0, 0)
			path.add(currentPosition)

			for(var i = 0; i < this.alphabet.length; i++){

					symbol = this.alphabet.charAt(i)
				
					switch(symbol){
							case('F'):
									// Move forward and draw line
									currentPosition += vector
									path.lineTo(currentPosition)
									break
							case('f'):
									// Move forward without drawing a line
									currentPosition += vector
									path = new Path()
									path.add(currentPosition)
									break
							case('+'):
									// Change angle
									vector.angle -= 360 / this.angle
									break
							case('-'):
									// Change angle
									vector.angle += 360 / this.angle
									break
							case('|'):
									// Turn around 180°
									vector.angle += 180
									break
							case('<'):
									// Increases length of line if lineChange > 1, otherwise decrease
									vector.length *= lineChange
									break
							case('>'):
									// Decreases length of line if lineChange > 1, otherwise increase
									vector.length /= lineChange
									break
							case('['):
									// Adds another branch
									branches.push([path, currentPosition, vector])
									vector = new Point(vector)
									path = new Path()
									path.add(currentPosition)
									break
							case(']'):
									// Returns  to the parrent branch
									branchArray = branches[branches.length -1]
									path = branchArray[0]
									currentPosition = branchArray[1]
									vector = branchArray[2]
									branches.pop()
									break				
					}
			}
}



var lsystem = new Lsystem(10, 'F++F++F++F++F')
lsystem.rules.push(new Rule('F', 'F++F++F|F-F++F'))
lsystem.draw(4, 10, 0.9)


/*
				

Shapes & Shape-tiling
		Pentaflake
				10, 'F++F++F++F++F'
				new Rule('F', 'F++F++F|F-F++F')
		Pentigree
				5, 'F-F-F-F-F'
				new Rule('F', 'F-F++F+F-F-F')
		Penrose
				10, '+WF--XF---YF--ZF'
				new Rule('W', 'YF++ZF----XF[-YF----WF]++')
				new Rule('X', '+YF--ZF[---WF--XF]+')
				new Rule('Y', '-WF++XF[+++YF++ZF]-')
				new Rule('Z', '--YF++++WF[+ZF++++XF]--XF')
				new Rule('F', ' ')
		Circular Tile
				24, 'X+X+X+X+X+X+X+X+X+X+X+X+X+X+X+X+X+X+X+X+X+X+X+X'
				new Rule('X', '[F+F+F+F[---X-Y]+++++F++++++++F-F-F-F]')
				new Rule('Y', '[F+F+F+F[---Y]+++++F++++++++F-F-F-F]')

Dragons
		Boundary
				4, 'OTUZ'
				new Rule('O', 'FO+F-T')
				new Rule('P', '++F--U+F-X')
				new Rule('Q', '-F+V++F--Q')
				new Rule('R', '-F+ZFS')
				new Rule('S', 'FW')
				new Rule('T', '++F--U')
				new Rule('U', '++F--Y')
				new Rule('V', 'FS')
				new Rule('W', 'FO+F-P')
				new Rule('X', '++F--Y+F-X')
				new Rule('Y', '-F+R++F--Q)')
				new Rule('Z', '-F+ZFW')
				new Rule('F', ' ')
		Cross
				4, 'FX'
				new Rule('X', 'FX+FX+FXFY-FY-')
				new Rule('Y', '+FX+FXFY-FY-FY')
				new Rule('F', ' ')
		Round
				8, 'X'
				new Rule('X', 'FX+FZ+FY')
				new Rule('Y', 'FX-FZ-FY')
				new Rule('Z', 'FZ')
				new Rule('F', ' ')

Misc.
		Church
				4, 'F'
				new Rule('F', 'F+FF-F-FF+F')
		Dekking's Church
				4, 'WZYX'
				new Rule('W', 'FW+F-XFW-F+Z')
				new Rule('X', '++F--Y-F+Z++F--Y-F+Z')
				new Rule('Y', '++F--Y+F-X')
				new Rule('Z', 'FW+F-X')
				new Rule('F', ' ')
		Daisy Chain
				8, 'F'
				new Rule('F', 'F+F-F-FF+F+F')
		Doily
				12, 'F--F--F--F--F--F'
				new Rule('F', '-F[--F--F]++F--F+')
		Hex Crystal
				12, 'F--F--F--F--F--F'
				new Rule('F', '-F[--F--F]++F[++F--F]--F+')
		Zig-Zag
				4, 'F'
				new Rule('F', 'F-F+F')
*/